ColorConfigurator = {}

local ColorConfigurator_mt = Class(ColorConfigurator)

function ColorConfigurator.new(baseDir, modSettingsDir, modName, customMt)
	local self = setmetatable({}, customMt or ColorConfigurator_mt)

	self.baseDirectory = baseDir
	self.customEnvironment = modName

	self.modSettings = {
		directory = modSettingsDir,
		xmlFilename = modSettingsDir .. "configs.xml"
	}

	createFolder(modSettingsDir)

	self.firstTimeRun = not Utils.getNoNil(getXMLBool(g_savegameXML, "gameSettings.colorConfigurator#initialized"), false)

	self.addGuiElements()

	ColorConfiguratorUtil.overwrittenFunction(GuiOverlay, "loadOverlay", self, "guiOverlay_loadOverlay")

	self.colorConfiguratorPickerDialog = ColorConfiguratorPickerDialog.new(nil, nil, self)
	self.colorConfiguratorDialog = ColorConfiguratorDialog.new(nil, nil, Utils.getFilename("materials.xml", baseDir), self)

	g_gui:loadProfiles(Utils.getFilename("gui/guiProfiles.xml", baseDir))
	g_gui:loadGui(g_gui.guis["ColorPickerDialog"].xmlFilename, "ColorConfiguratorPickerDialog", self.colorConfiguratorPickerDialog)
	g_gui:loadGui(Utils.getFilename("gui/ColorConfiguratorDialog.xml", baseDir), "ColorConfiguratorDialog", self.colorConfiguratorDialog)

	self.supportedConfigurations = {"baseMaterial", "designMaterial", "baseColor", "designColor", "wrappingColor", "rimColor"}

	for i = 2, 49 do
		table.insert(self.supportedConfigurations, "designMaterial" .. i)
	end

	self.additionalConfigs = self:loadAdditionalConfigs(true)
	addConsoleCommand("ccRemoveAddConfigs", "", "consoleCommandRemoveAddConfigs", self)

	ColorConfiguratorUtil.overwrittenFunction(g_gui, "showColorPickerDialog", self, "g_gui_showColorPickerDialog")
	ColorConfiguratorUtil.prependedFunction(TypeManager, "finalizeTypes", self, "typeManager_finalizeTypes")
	ColorConfiguratorUtil.appendedFunction(ConfigurationUtil, "getConfigColorPostLoad", self, "configurationUtil_getConfigColorPostLoad")
	ColorConfiguratorUtil.overwrittenStaticFunction(ConfigurationUtil, "getMaterialByConfigId", self, "configurationUtil_getMaterialByConfigId")
	ColorConfiguratorUtil.overwrittenStaticFunction(ConfigurationUtil, "getColorByConfigId", self, "configurationUtil_getColorByConfigId")
	ColorConfiguratorUtil.overwrittenStaticFunction(ConfigurationUtil, "applyConfigMaterials", self, "configurationUtil_applyConfigMaterials")
	ColorConfiguratorUtil.overwrittenStaticFunction(LicensePlateManager, "readLicensePlateData", self, "licensePlateManager_readLicensePlateData")
	ColorConfiguratorUtil.appendedFunction(LicensePlateManager, "writeLicensePlateData", self, "licensePlateManager_writeLicensePlateData")
	ColorConfiguratorUtil.prependedFunction(VehicleLoadingUtil, "loadVehiclesAtPlaceStepFinished", self, "vehicleLoadingUtil_loadVehiclesAtPlaceStepFinished")
	ColorConfiguratorUtil.appendedFunction(ShopConfigScreen, "onPickColor", self, "shopConfigScreen_onPickColor")
	ColorConfiguratorUtil.overwrittenFunction(ShopConfigScreen, "onChangeLicensePlate", self, "shopConfigScreen_onChangeLicensePlate")
	ColorConfiguratorUtil.overwrittenFunction(ShopConfigScreen, "getConfigurationCostsAndChanges", self, "shopConfigScreen_getConfigurationCostsAndChanges")
	ColorConfiguratorUtil.overwrittenFunction(ShopConfigScreen, "setStoreItem", self, "shopConfigScreen_setStoreItem")
	ColorConfiguratorUtil.overwrittenFunction(ShopConfigScreen, "onVehicleLoaded", self, "shopConfigScreen_onVehicleLoaded")
	ColorConfiguratorUtil.prependedFunction(ShopConfigScreen, "updateData", self, "shopConfigScreen_updateData")
	ColorConfiguratorUtil.overwrittenFunction(WorkshopScreen, "setConfigurations", self, "workshopScreen_setConfigurations")
	ColorConfiguratorUtil.overwrittenFunction(ChangeVehicleConfigEvent, "run", self, "changeVehicleConfigEvent_run")

	return self
end

function ColorConfigurator.addGuiElements()
	local mapping = Gui.CONFIGURATION_CLASS_MAPPING

	mapping.colorConfiguratorRender = ColorConfiguratorRenderElement
	mapping.colorConfiguratorSlider = ColorConfiguratorSliderElement
	mapping.colorConfiguratorPicker = ColorConfiguratorPickerElement
end

function ColorConfigurator:saveAdditionalConfigs(printToLog)
	local xmlFile = XMLFile.create("configsXML", self.modSettings.xmlFilename, "vehicles")

	if xmlFile ~= nil then
		local i = 0

		for name, configs in pairs(self.additionalConfigs) do
			local vehicleKey = string.format("vehicles.vehicle(%d)", i)

			xmlFile:setString(vehicleKey .. "#filename", HTMLUtil.encodeToHTML(NetworkUtil.convertToNetworkFilename(name)))

			local j = 0

			for configName, configData in pairs(configs) do
				local configsKey = string.format("%s.configs(%s)", vehicleKey, j)

				xmlFile:setString(configsKey .. "#name", configName)

				local k = 0

				for _, data in pairs(configData) do
					local configKey = string.format("%s.config(%s)", configsKey, k)

					xmlFile:setString(configKey .. "#name", data.name)
					xmlFile:setVector(configKey .. "#color", data.color)
					xmlFile:setInt(configKey .. "#material", data.material)

					k = k + 1
				end

				j = j + 1
			end

			i = i + 1
		end

		xmlFile:save(printToLog)
		xmlFile:delete()
	end
end

function ColorConfigurator:loadAdditionalConfigs(printToLog)
	local additionalConfigs = {}
	local xmlFile = XMLFile.loadIfExists("configsXML", self.modSettings.xmlFilename, "vehicles")

	if xmlFile ~= nil then
		xmlFile:iterate("vehicles.vehicle", function (_, vehicleKey)
			local filename = NetworkUtil.convertFromNetworkFilename(xmlFile:getString(vehicleKey .. "#filename"))

			if additionalConfigs[filename] == nil then
				additionalConfigs[filename] = {}
			end

			xmlFile:iterate(vehicleKey .. ".configs", function (_, configsKey)
				local configName = xmlFile:getString(configsKey .. "#name")

				if additionalConfigs[filename][configName] == nil then
					additionalConfigs[filename][configName] = {}
				end

				xmlFile:iterate(configsKey .. ".config", function (_, configKey)
					local name = xmlFile:getString(configKey .. "#name")
					local color = xmlFile:getVector(configKey .. "#color")
					local material = xmlFile:getInt(configKey .. "#material")

					table.insert(additionalConfigs[filename][configName], {
						name = name,
						color = color,
						material = material,
						isAddConfig = true
					})
				end)
			end)
		end)

		if printToLog then
			ColorConfiguratorUtil.printToLog(xmlFile)
		end

		xmlFile:delete()
	end

	return additionalConfigs
end

function ColorConfigurator:addAdditionalConfig(filename, configName, configData, doNotSave)
	if filename == nil or configName == nil and configData == nil then
		return false
	end

	if self.additionalConfigs[filename] == nil then
		self.additionalConfigs[filename] = {}
	end

	if self.additionalConfigs[filename][configName] == nil then
		self.additionalConfigs[filename][configName] = {}
	end

	table.insert(self.additionalConfigs[filename][configName], configData)

	if not doNotSave then
		self:saveAdditionalConfigs()
	end

	return true
end

function ColorConfigurator:removeAdditionalConfig(filename, configName, configData)
	if filename == nil or configName == nil or configData == nil then
		return false
	end

	if self.additionalConfigs[filename] ~= nil and self.additionalConfigs[filename][configName] ~= nil then
		local success = table.removeElement(self.additionalConfigs[filename][configName], configData)

		if success then
			if next(self.additionalConfigs[filename][configName]) == nil then
				self.additionalConfigs[filename][configName] = nil
			end

			if next(self.additionalConfigs[filename]) == nil then
				self.additionalConfigs[filename] = nil
			end

			self:saveAdditionalConfigs()
		end
	end
end

function ColorConfigurator:getAdditionalConfigs(filename, configName)
	if filename == nil or configName == nil then
		return
	end

	if self.additionalConfigs[filename] ~= nil and self.additionalConfigs[filename][configName] ~= nil then
		return self.additionalConfigs[filename][configName]
	end
end

function ColorConfigurator:consoleCommandRemoveAddConfigs(verbose)
	verbose = Utils.stringToBoolean(verbose)

	local ret = "Use 'true' parameter to remove all configurations."

	if verbose then
		table.clear(self.additionalConfigs)
		self:saveAdditionalConfigs()

		ret = "All configurations removed."
	end

	return ret
end

function ColorConfigurator:setColor(colors)
	local shopConfigScreen = self.colorConfiguratorPickerDialog.target
	local configName = self.colorConfiguratorPickerDialog.args.configName
	local colorOptionIndex = self.colorConfiguratorPickerDialog.args.colorOptionIndex
	local xmlFilename = shopConfigScreen.storeItem.xmlFilename

	local configItems = shopConfigScreen.storeItem.configurations[configName]
	local configItem = nil

	for j, item in pairs(configItems) do
		if item.isCustomConfig then
			configItem = item
			break
		end
	end

	if configItem == nil then
		Logging.warning("Cannot find custom configuration '%s' for '%s',", configName, xmlFilename)
		return
	end

	if shopConfigScreen.licensePlateData == nil then
		shopConfigScreen.licensePlateData = {}
	end

	if shopConfigScreen.licensePlateData.colorConfiguratorData == nil then
		shopConfigScreen.licensePlateData.colorConfiguratorData = {}
	end

	shopConfigScreen.licensePlateData.colorConfiguratorData[configName] = {
		color = {colors[1], colors[2], colors[3]},
		material = colors[4]
	}

	shopConfigScreen:onPickColor(configItem.index, {
		configName = configName,
		colorOptionIndex = colorOptionIndex
	}, false)
end

function ColorConfigurator.g_gui_showColorPickerDialog(colorConfigurator, self, superFunc, args)
	local dialogName = "ColorPickerDialog"

	if args ~= nil and args.args ~= nil and args.target == g_shopConfigScreen and args.callback == g_shopConfigScreen.onPickColor then
		for _, configName in pairs(colorConfigurator.supportedConfigurations) do
			if configName == args.args.configName then
				dialogName = "ColorConfiguratorPickerDialog"
				break
			end
		end
	end

	local dialog = self:showDialog(dialogName)

	if dialog ~= nil and args ~= nil then
		dialog.target:setCallback(args.callback, args.target, args.args)
		dialog.target:setColors(args.colors, args.defaultColor, args.defaultColorMaterial)
	end

	if colorConfigurator.firstTimeRun and dialogName == "ColorConfiguratorPickerDialog" then
		local mod = g_modManager:getModByName(colorConfigurator.customEnvironment)
		local text = string.format("%s\n\n%s", utf8ToUpper(mod.title), g_i18n:getText("colorConfigurator_ui_info"))

		self:showInfoDialog({
			dialogType = DialogElement.TYPE_INFO,
			text = text
		})

		setXMLBool(g_savegameXML, "gameSettings.colorConfigurator#initialized", true)
		g_gameSettings:saveToXMLFile(g_savegameXML)

		colorConfigurator.firstTimeRun = false
	end
end

function ColorConfigurator.typeManager_finalizeTypes(colorConfigurator, self)
	if self.typeName == "vehicle" then
		for typeName, typeEntry in pairs(self:getTypes()) do
			self:addSpecialization(typeName, colorConfigurator.customEnvironment .. ".vehicleColor")
		end
	end
end

function ColorConfigurator.configurationUtil_getConfigColorPostLoad(colorConfigurator, xmlFile, baseKey, baseDir, customEnvironment, isMod, configurationItems, storeItem)
	if #configurationItems > 1 then
		local selectableItems = 0

		for i, item in ipairs(configurationItems) do
			if item.isSelectable ~= false then
				selectableItems = selectableItems + 1
			end
		end

		if selectableItems > 1 then
			local addItem = false

			for _, configName in pairs(colorConfigurator.supportedConfigurations) do
				if baseKey == string.format("vehicle.%sConfigurations", configName) then
					addItem = true
					break
				end
			end

			if addItem then
				local configItem = StoreItemUtil.addConfigurationItem(configurationItems, g_i18n:getText("colorConfigurator_ui_customColorName"), "", 0, 0, false)

				configItem.color = {1, 0, 1, 0}
				configItem.isCustomConfig = true
			end
		end
	end
end

function ColorConfigurator.configurationUtil_getMaterialByConfigId(colorConfigurator, superFunc, object, configName, configId)
	if configId ~= nil and object.getCustomConfigurationByName ~= nil then
		local customConfig = object:getCustomConfigurationByName(configName)

		if customConfig ~= nil then
			return customConfig.material
		end
	end

	return superFunc(object, configName, configId)
end

function ColorConfigurator.configurationUtil_getColorByConfigId(colorConfigurator, superFunc, object, configName, configId)
	if configId ~= nil and object.getCustomConfigurationByName ~= nil then
		local customConfig = object:getCustomConfigurationByName(configName)

		if customConfig ~= nil then
			local r, g, b = unpack(customConfig.color)

			return {
				r,
				g,
				b,
				customConfig.material
			}
		end
	end

	return superFunc(object, configName, configId)
end

function ColorConfigurator.configurationUtil_applyConfigMaterials(colorConfigurator, superFunc, object, xmlFile, configName, configId)
	local customConfigFound = false

	if configId ~= nil and object.getCustomConfigurationByName ~= nil then
		local customConfig = object:getCustomConfigurationByName(configName)

		if customConfig ~= nil then
			local customConfigFound = true
			local storeItem = g_storeManager:getItemByXMLFilename(object.configFileName)

			if storeItem ~= nil then
				local defaultConfigId = StoreItemUtil.getDefaultConfigId(storeItem, configName)
				local configuration = g_configurationManager:getConfigurationDescByName(configName)
				local xmlKey = configuration.xmlKey

				if xmlKey ~= nil then
					xmlKey = "." .. xmlKey
				else
					xmlKey = ""
				end

				local configKey = string.format("vehicle%s.%sConfigurations.%sConfiguration(%d)", xmlKey, configName, configName, defaultConfigId - 1)

				if xmlFile:hasProperty(configKey) then
					xmlFile:iterate(configKey .. ".material", function (_, key)
						local baseMaterialNode = xmlFile:getValue(key .. "#node", nil, object.components, object.i3dMappings)
						local refMaterialNode = xmlFile:getValue(key .. "#refNode", nil, object.components, object.i3dMappings)

						if baseMaterialNode ~= nil and refMaterialNode ~= nil then
							local oldMaterial = getMaterial(baseMaterialNode, 0)
							local newMaterial = getMaterial(refMaterialNode, 0)

							for _, component in pairs(object.components) do
								ConfigurationUtil.replaceMaterialRec(object, component.node, oldMaterial, newMaterial)
							end
						end

						local materialName = xmlFile:getValue(key .. "#name")

						if materialName ~= nil then
							local shaderParameterName = xmlFile:getValue(key .. "#shaderParameter")

							if shaderParameterName ~= nil then
								if object.setBaseMaterialColor ~= nil then
									object:setBaseMaterialColor(materialName, shaderParameterName, customConfig.color, customConfig.material)
								end
							end
						end
					end)
				end
			end
		end
	end

	if not customConfigFound then
		superFunc(object, xmlFile, configName, configId)
	end
end

function ColorConfigurator.licensePlateManager_readLicensePlateData(colorConfigurator, superFunc, streamId, connection)
	local licensePlateData = superFunc(streamId, connection)

	local hasColorConfiguratorData = streamReadBool(streamId)

	if hasColorConfiguratorData then
		local numConfigurations = streamReadUInt8(streamId)

		licensePlateData.colorConfiguratorData = {}

		for _ = 1, numConfigurations do
			local configName = streamReadString(streamId)
			local r = streamReadFloat32(streamId)
			local g = streamReadFloat32(streamId)
			local b = streamReadFloat32(streamId)
			local material = streamReadInt32(streamId)

			licensePlateData.colorConfiguratorData[configName] = {
				color = ColorConfiguratorUtil.formatColor(r, g, b),
				material = material
			}
		end
	end

	return licensePlateData
end

function ColorConfigurator.licensePlateManager_writeLicensePlateData(colorConfigurator, streamId, connection, licensePlateData)
	if streamWriteBool(streamId, licensePlateData ~= nil and licensePlateData.colorConfiguratorData ~= nil) then
		local numConfigurations = 0

		for _, _ in pairs(licensePlateData.colorConfiguratorData) do
			numConfigurations = numConfigurations + 1
		end

		streamWriteUInt8(streamId, numConfigurations)

		for configName, configData in pairs(licensePlateData.colorConfiguratorData) do
			streamWriteString(streamId, configName)
			streamWriteFloat32(streamId, configData.color[1])
			streamWriteFloat32(streamId, configData.color[2])
			streamWriteFloat32(streamId, configData.color[3])
			streamWriteInt32(streamId, configData.material)
		end
	end
end

function ColorConfigurator.vehicleLoadingUtil_loadVehiclesAtPlaceStepFinished(colorConfigurator, _, vehicle, vehicleLoadState, arguments)
	local vehiclesToLoad = arguments.vehiclesToLoad

	if vehicle ~= nil and vehiclesToLoad.licensePlateData ~= nil and vehicle.setCustomConfigurationsData ~= nil then
		vehicle:setCustomConfigurationsData(vehiclesToLoad.licensePlateData.colorConfiguratorData)
	end
end

function ColorConfigurator.shopConfigScreen_onPickColor(colorConfigurator, self, colorIndex, args, noUpdate)
	if colorIndex ~= nil then
		local configName = args.configName
		local isCustomConfig = self.storeItem.configurations[configName][colorIndex].isCustomConfig

		if isCustomConfig then
			local colorConfiguratorData = self.licensePlateData.colorConfiguratorData

			if colorConfiguratorData ~= nil then
				local customConfig = colorConfiguratorData[configName]

				if customConfig ~= nil then
					local colorOptionIndex = args.colorOptionIndex
					local element = self.colorElements[colorOptionIndex]
					local color = customConfig.color
					local material = customConfig.material
					local isMetallic = ConfigurationUtil.isColorMetallic(material)

					element:getDescendantByName("colorImage"):setImageColor(nil, unpack(color))
					element:getDescendantByName("colorImage"):setVisible(not isMetallic)
					element:getDescendantByName("colorImageMetallic"):setImageColor(nil, unpack(color))
					element:getDescendantByName("colorImageMetallic"):setVisible(isMetallic)
				end
			end
		end
	end
end

function ColorConfigurator.shopConfigScreen_onChangeLicensePlate(colorConfigurator, self, superFunc, licensePlateData)
	if licensePlateData ~= nil and self.licensePlateData.colorConfiguratorData ~= nil then
		licensePlateData.colorConfiguratorData = self.licensePlateData.colorConfiguratorData
	end

	superFunc(self, licensePlateData)
end

function ColorConfigurator.shopConfigScreen_getConfigurationCostsAndChanges(colorConfigurator, self, superFunc, storeItem, vehicle, saleItem)
	local basePrice, upgradePrice, hasChanges = superFunc(self, storeItem, vehicle, saleItem)

	if not hasChanges and vehicle ~= nil then
		if self.licensePlateData ~= nil then
			if vehicle.getCustomConfigurationsDataIsEqual ~= nil then
				if not vehicle:getCustomConfigurationsDataIsEqual(self.licensePlateData.colorConfiguratorData) then
					hasChanges = true
				end
			end
		end
	end

	return basePrice, upgradePrice, hasChanges
end

function ColorConfigurator.shopConfigScreen_setStoreItem(colorConfigurator, self, superFunc, storeItem, vehicle, saleItem, configBasePrice, configurations)
	self:deletePreviewVehicles()

	self.storeItem = storeItem
	self.vehicle = vehicle
	self.saleItem = saleItem
	self.configBasePrice = Utils.getNoNil(configBasePrice, 0)

	if configurations == nil then
		configurations = {}

		if storeItem.defaultConfigurationIds ~= nil then
			for configName, index in pairs(storeItem.defaultConfigurationIds) do
				configurations[configName] = index
			end
		end

		if vehicle ~= nil and vehicle.configurations ~= nil then
			for configName, index in pairs(vehicle.configurations) do
				configurations[configName] = index
			end
		end

		if saleItem ~= nil and saleItem.boughtConfigurations ~= nil then
			for configName, boughtItems in pairs(saleItem.boughtConfigurations) do
				for index, value in pairs(boughtItems) do
					if value then
						configurations[configName] = index
					end
				end
			end
		end
	end

	self.configurations = configurations
	self.subConfigurations = {}
	self.currentConfigSet = 1
	self.isLoadingInitial = true
	self.licensePlateData = {}

	if g_licensePlateManager:getAreLicensePlatesAvailable() then
		if vehicle ~= nil then
			if vehicle.getLicensePlatesData ~= nil then
				local data = vehicle:getLicensePlatesData()

				if data ~= nil and data.characters ~= nil then
					self.licensePlateData = {
						variation = data.variation,
						colorIndex = data.colorIndex,
						placementIndex = data.placementIndex,
						characters = table.copy(data.characters)
					}
					local defaultPlacementIndex, hasFrontPlate = vehicle:getLicensePlateDialogSettings()
					self.licensePlateData.defaultPlacementIndex = defaultPlacementIndex
					self.licensePlateData.hasFrontPlate = hasFrontPlate
				else
					self.licensePlateData = g_currentMission:getLastCreatedLicensePlate() or g_licensePlateManager:getRandomLicensePlateData()
				end
			end
		else
			self.licensePlateData = g_currentMission:getLastCreatedLicensePlate() or g_licensePlateManager:getRandomLicensePlateData()
		end
	end

	if vehicle ~= nil and vehicle.getCustomConfigurations ~= nil then
		local colorConfiguratorData = vehicle:getCustomConfigurations()

		if colorConfiguratorData ~= nil then
			self.licensePlateData.colorConfiguratorData = table.copy(colorConfiguratorData, math.huge)
		end
	end

	self:processStoreItemConfigurations(storeItem, vehicle, saleItem)
	self:updateDisplay(storeItem, vehicle, saleItem)
	self:resetCamera()
end

function ColorConfigurator.shopConfigScreen_onVehicleLoaded(colorConfigurator, self, superFunc, vehicle, loadingState, asyncArguments)
	if loadingState == VehicleLoadingUtil.VEHICLE_LOAD_OK then
		if asyncArguments.openCounter == self.openCounter then
			if self.isLoadingInitial or self.isOpen then
				if self.licensePlateData ~= nil and vehicle.setCustomConfigurationsData ~= nil then
					vehicle:setCustomConfigurationsData(self.licensePlateData.colorConfiguratorData)
				end
			end
		end
	end

	superFunc(self, vehicle, loadingState, asyncArguments)
end

function ColorConfigurator.shopConfigScreen_updateData(colorConfigurator, self, storeItem, vehicle, saleItem)
	if self.licensePlateData ~= nil and self.licensePlateData.colorConfiguratorData ~= nil then
		for configName, configData in pairs(self.licensePlateData.colorConfiguratorData) do
			local id = self.configurations[configName]
			local config = storeItem.configurations[configName][id]

			if not config.isCustomConfig and self.licensePlateData.colorConfiguratorData[configName] ~= nil then
				self.licensePlateData.colorConfiguratorData[configName] = nil
			end
		end
	end
end

function ColorConfigurator.workshopScreen_setConfigurations(colorConfigurator, self, superFunc, vehicle, buyItem, storeItem, configs, price, licensePlateData)
	self.vehicle = vehicle

	self.shopConfigScreen:onClickBack()

	if not buyItem and storeItem ~= nil and configs ~= nil then
		local areChangesMade = false
		local newConfigs = {}

		for configName, configValue in pairs(configs) do
			if self.vehicle.configurations[configName] ~= configValue then
				newConfigs[configName] = configs[configName]
				areChangesMade = true
			end
		end

		if self.vehicle.getLicensePlatesDataIsEqual ~= nil and not self.vehicle:getLicensePlatesDataIsEqual(licensePlateData) then
			areChangesMade = true
		end

		if licensePlateData ~= nil and self.vehicle.getCustomConfigurationsDataIsEqual ~= nil and not self.vehicle:getCustomConfigurationsDataIsEqual(licensePlateData.colorConfiguratorData) then
			areChangesMade = true
		end

		if areChangesMade then
			if not g_currentMission.controlPlayer and g_currentMission.controlledVehicle ~= nil and self.vehicle == g_currentMission.controlledVehicle then
				g_currentMission:onLeaveVehicle()
			end

			g_client:getServerConnection():sendEvent(ChangeVehicleConfigEvent.new(self.vehicle, price, g_currentMission:getFarmId(), newConfigs, licensePlateData))
		end
	else
		self:onClickBack()

		if self.owner ~= nil then
			self.owner:openMenu()
		end
	end
end

function ColorConfigurator.changeVehicleConfigEvent_run(colorConfigurator, self, superFunc, connection)
	if not connection:getIsServer() then
		local success = false
		local vehicle = self.vehicle

		if vehicle ~= nil and vehicle.isVehicleSaved and not vehicle.isControlled and g_currentMission:getHasPlayerPermission("buyVehicle", connection) then
			for configName, configId in pairs(self.configurations) do
				ConfigurationUtil.addBoughtConfiguration(vehicle, configName, configId)
				ConfigurationUtil.setConfiguration(vehicle, configName, configId)
			end

			vehicle:setLicensePlatesData(self.licensePlateData)

			if self.licensePlateData ~= nil and vehicle.setCustomConfigurationsData ~= nil then
				vehicle:setCustomConfigurationsData(self.licensePlateData.colorConfiguratorData)
			end

			vehicle.isReconfigurating = true

			g_server:broadcastEvent(VehicleSetIsReconfiguratingEvent.new(vehicle), nil, nil, vehicle)

			local xmlFile = Vehicle.getReloadXML(vehicle)
			local key = "vehicles.vehicle(0)"

			local function asyncCallbackFunction(_, newVehicle, vehicleLoadState, arguments)
				if vehicleLoadState == VehicleLoadingUtil.VEHICLE_LOAD_OK then
					success = true

					g_currentMission:addMoney(-self.price, self.farmId, MoneyType.SHOP_VEHICLE_BUY, true)
					vehicle:removeFromPhysics()
					g_currentMission:removeVehicle(vehicle)
				else
					g_currentMission:removeVehicle(newVehicle)
					vehicle:addToPhysics()
				end

				xmlFile:delete()
				connection:sendEvent(ChangeVehicleConfigEvent.newServerToClient(success))
			end

			VehicleLoadingUtil.loadVehicleFromSavegameXML(xmlFile, key, false, false, nil, nil, asyncCallbackFunction, nil, {})
		else
			connection:sendEvent(ChangeVehicleConfigEvent.newServerToClient(false))
		end

		return
	end

	g_workshopScreen:onVehicleChanged(self.successful)
end

function ColorConfigurator.guiOverlay_loadOverlay(colorConfigurator, self, superFunc, overlay, overlayName, imageSize, profile, xmlFile, key)
	local overlay = superFunc(self, overlay, overlayName, imageSize, profile, xmlFile, key)

	if overlay ~= nil and overlay.filename == "g_colorConfiguratorUIFilename" then
		overlay.filename = Utils.getFilename("gui/ui_elements.png", colorConfigurator.baseDirectory)
	end

	return overlay
end

g_colorConfigurator = ColorConfigurator.new(g_currentModDirectory, g_currentModSettingsDirectory, g_currentModName)