ColorConfiguratorDialog = {
	CONTROLS = {
		"multiMaterialType",
		"textInputRed",
		"textInputGreen",
		"textInputBlue",
		"sliderRed",
		"sliderGreen",
		"sliderBlue",
		"sceneRender",
		"tooltipLayout",
		"glyphBox",
		"loadingAnimation",
		"saveButton",
		"multiColorMode",
		"colorPicker",
		"textInputHue",
		"textInputSaturation",
		"textInputValue",
		"pickedColor"
	},
	COLOR = {
		RED = 1,
		GREEN = 2,
		BLUE = 3,
		MATERIAL = 4
	},
	L10N_SYMBOL = {
		COPY_COLOR = "colorConfigurator_toolTip_colorCopied",
		PASTE_COLOR = "colorConfigurator_toolTip_colorPasted",
		NO_MATERIAL = "colorConfigurator_toolTip_noMaterial",
		COLOR_MODE = "colorConfigurator_button_mode"
	},
	MAX_CONFIGS = 128,
	MODE_GIANTS = 1,
	MODE_RGB = 2
}

local ColorConfiguratorDialog_mt = Class(ColorConfiguratorDialog, MessageDialog)

local function NO_CALLBACK()
end

function ColorConfiguratorDialog.new(target, custom_mt, materialsXMLFilename, colorConfigurator)
	local self = MessageDialog.new(target, custom_mt or ColorConfiguratorDialog_mt)

	self:registerControls(ColorConfiguratorDialog.CONTROLS)

	self.colorConfigurator = colorConfigurator
	self.colorConfiguratorPickerDialog = colorConfigurator.colorConfiguratorPickerDialog

	self.settingsXMLFilename = colorConfigurator.modSettings.directory .. "settings.xml"

	self.materials = self:loadMaterials(materialsXMLFilename, false)
	self.colors = {1, 1, 1, 0}

	self.showGlyphs = true

	self.isRGBMode = true
	self.savedMode = true

	self.lastInputHelpMode = nil
	self.sharedLoadRequestId = nil
	self.copiedColors = nil

	self.sliderSample = GuiSoundPlayer.SOUND_SAMPLES.HOVER
	self.customTextSample = false

	return self
end

function ColorConfiguratorDialog:loadMaterials(xmlFilename, printToLog)
	local materials = {}
	local xmlFile = XMLFile.load("materialsXML", xmlFilename)

	if xmlFile ~= nil then
		xmlFile:iterate("materials.material", function (_, key)
			local name = xmlFile:getString(key .. "#name")
			local shaderId = xmlFile:getInt(key .. "#shaderId")

			if name ~= nil and name:len() > 0 and shaderId ~= nil then
				materials[#materials + 1] = {
					name = name,
					shaderId = shaderId
				}
			end
		end)

		if printToLog then
			ColorConfiguratorUtil.printToLog(xmlFile)
		end

		xmlFile:delete()
	end

	return materials
end

function ColorConfiguratorDialog:delete()
	if self.sharedLoadRequestId ~= nil then
		g_i3DManager:releaseSharedI3DFile(self.sharedLoadRequestId)
		self.sharedLoadRequestId = nil
	end

	ColorConfiguratorDialog:superClass().delete(self)
end

function ColorConfiguratorDialog:onGuiSetupFinished()
	ColorConfiguratorDialog:superClass().onGuiSetupFinished(self)

	self:loadSettingsFromXMLFile(true)

	self.textInputElements = {self.textInputRed, self.textInputGreen, self.textInputBlue}
	self.textInputHSVElements = {self.textInputHue, self.textInputSaturation, self.textInputValue}
	self.sliderElements = {self.sliderRed, self.sliderGreen, self.sliderBlue}

	for _, elem in pairs(self.sliderElements) do
		ColorConfiguratorUtil.overwrittenFunction(elem, "setValue", self, "sliderElement_setValue")
	end

	if not self.customTextSample then
		for _, elem in pairs(self.textInputElements) do
			elem.customFocusSample = nil
		end
		for _, elem in pairs(self.textInputHSVElements) do
			elem.customFocusSample = nil
		end
	end

	local materialsTexts = {}

	for i, mat in pairs(self.materials) do
		table.insert(materialsTexts, string.format("%s (%d)", g_i18n:getText(string.format("colorConfigurator_material_%s", mat.name)), i))
	end

	self.multiMaterialType:setTexts(materialsTexts)

	self.sceneRender.filename = string.gsub(self.sceneRender.filename, "$moddir$/", self.colorConfigurator.baseDirectory)

	self.glyphBox:setVisible(self.showGlyphs)

	self:setRGBMode(self.isRGBMode)
end

function ColorConfiguratorDialog:onCreate(element)
	ColorConfiguratorDialog:superClass().onCreate(self, element)

	self.colorPicker:updatePointerLimits()
	self:updateHSVElements()
end

function ColorConfiguratorDialog:sliderElement_setValue(slider, superFunc, newValue, doNotUpdateDataElement, immediateMode)
	if not ColorConfiguratorUtil.getIsSliderChangeAllowed(slider, immediateMode) then
		return false
	end

	if self.isRGBMode then
		return superFunc(slider, newValue, doNotUpdateDataElement, immediateMode)
	end

	slider.stepSize = 0.0001

	local retValue = superFunc(slider, newValue, doNotUpdateDataElement, immediateMode)

	slider.stepSize = 0.003921

	return retValue
end

function ColorConfiguratorDialog:update(dt)
	ColorConfiguratorDialog:superClass().update(self, dt)

	if self.showGlyphs then
		local inputHelpMode = g_inputBinding:getInputHelpMode()

		if inputHelpMode ~= self.lastInputHelpMode then
			self.lastInputHelpMode = inputHelpMode

			self.glyphBox:setVisible(inputHelpMode ~= GS_INPUT_HELP_MODE_GAMEPAD)

			self.glyphBox.elements[1]:setActions({
				InputAction.AXIS_MAP_ZOOM_IN,
				InputAction.AXIS_MAP_ZOOM_OUT
			})

			self.glyphBox.elements[2]:setActions({
				InputAction.AXIS_LOOK_LEFTRIGHT_DRAG,
				InputAction.AXIS_LOOK_UPDOWN_DRAG
			})

			self.glyphBox.elements[3]:setActions({
				InputAction.MOUSE_ALT_COMMAND3_BUTTON
			})
		end
	end
end

function ColorConfiguratorDialog:registerActionEvents()
	g_inputBinding:registerActionEvent(InputAction.COLOR_CONFIGURATOR_COPY_COLOR, self, self.onCopyColorValues, false, true, false, true)
	g_inputBinding:registerActionEvent(InputAction.COLOR_CONFIGURATOR_PASTE_COLOR, self, self.onPasteColorValues, false, true, false, true)
end

function ColorConfiguratorDialog:removeActionEvents()
	g_inputBinding:removeActionEventsByTarget(self)
end

function ColorConfiguratorDialog:onClickOk()
	self:close()
	self.colorConfigurator:setColor(self.colors)
end

function ColorConfiguratorDialog:onClickSave()
	for _, elem in pairs(self.textInputElements) do
		if elem.isCapturingInput then
			return
		end
	end

	for _, elem in pairs(self.textInputHSVElements) do
		if elem.isCapturingInput then
			return
		end
	end

	local alreadyExists, limitReached = self:getCanAddConfig()

	if limitReached or alreadyExists then
		local dialogText = "colorConfigurator_ui_alreadyExists"

		if limitReached then
			dialogText = "colorConfigurator_ui_limitReached"
		end

		g_gui:showInfoDialog({
			text = g_i18n:getText(dialogText),
			dialogType = DialogElement.TYPE_INFO,

		})

		return
	end

	g_gui:showTextInputDialog({
		defaultText = g_i18n:getText("colorConfigurator_ui_defaultColorName"),
		dialogPrompt = g_i18n:getText("colorConfigurator_ui_enterColorName"),
		imePrompt = g_i18n:getText("colorConfigurator_ui_enterColorName"),
		confirmText = g_i18n:getText("button_save"),
		maxCharacters = 30,
		callback = function (result, yes)
			if yes then
				local storeItem = self.colorConfiguratorPickerDialog.target.storeItem
				local configName = self.colorConfiguratorPickerDialog.args.configName
				local r, g, b, mat = unpack(self.colors)

				local data = {
					name = result,
					material = mat,
					color = {r, g, b},
					isAddConfig = true
				}

				if storeItem.bundleInfo ~= nil then
					for _, bundleItem in ipairs(storeItem.bundleInfo.bundleItems) do
						self.colorConfigurator:addAdditionalConfig(bundleItem.xmlFilename, configName, data, true)
					end
				end

				self.colorConfigurator:addAdditionalConfig(storeItem.xmlFilename, configName, data)

				g_gui:showInfoDialog({
					text = string.format(g_i18n:getText("colorConfigurator_ui_colorAddedToList"), data.name),
					dialogType = DialogElement.TYPE_INFO,
					callback = function ()
						self:onClickBack()
					end
				})
			end
		end,
	})
end

function ColorConfiguratorDialog:onClickBack()
	ColorConfiguratorDialog:superClass().onClickBack(self)

	local colorConfiguratorPickerDialog = self.colorConfiguratorPickerDialog

	colorConfiguratorPickerDialog.disableOpenSound = true

	g_gui:showDialog(colorConfiguratorPickerDialog.name)

	colorConfiguratorPickerDialog:setColors(unpack(colorConfiguratorPickerDialog.previousColors))
	colorConfiguratorPickerDialog.disableOpenSound = false
end

function ColorConfiguratorDialog:onClickColorMode(state)
	local isRGBMode = state == ColorConfiguratorDialog.MODE_RGB

	self:setRGBMode(isRGBMode)
end

function ColorConfiguratorDialog:onOpen()
	ColorConfiguratorDialog:superClass().onOpen(self)

	if FocusManager:getFocusedElement() == g_gui.guis[self.name] then
		self:setSoundSuppressed(true)
		FocusManager:setFocus(self.multiMaterialType)
		self:setSoundSuppressed(false)
	end

	if self.sharedLoadRequestId == nil then
		self.loadingAnimation:setVisible(true)
		self.sharedLoadRequestId = g_i3DManager:loadSharedI3DFileAsync(self.sceneRender.filename, false, false, NO_CALLBACK, self, nil)
	end

	self.sceneRender:createScene()

	self:registerActionEvents()
end

function ColorConfiguratorDialog:onClose()
	self:removeActionEvents()

	self.sceneRender:destroyScene()
	self:updateTooltipLayout(false)

	if self.savedMode ~= self.isRGBMode then
		self:saveSettingsToXMLFile()
		self.savedMode = self.isRGBMode
	end

	ColorConfiguratorDialog:superClass().onClose(self)
end

function ColorConfiguratorDialog:onRenderLoad(scene, overlay)
	self:updateSceneRender()

	if self.loadingAnimation:getIsVisible() then
		self.loadingAnimation:setVisible(false)
	end
end

function ColorConfiguratorDialog:onClickMaterialType(state)
	self.colors[ColorConfiguratorDialog.COLOR.MATERIAL] = self.materials[state].shaderId

	self:updateSceneRender()
	self:updateTooltipLayout(false)
end

function ColorConfiguratorDialog:onButtonPressed(element, dismissal)
	self:verifyInputText(element, dismissal)
end

function ColorConfiguratorDialog:onTextChangedRed(element)
	self:onRGBElementChanged(ColorConfiguratorDialog.COLOR.RED, element.text, true)
end

function ColorConfiguratorDialog:onTextChangedGreen(element)
	self:onRGBElementChanged(ColorConfiguratorDialog.COLOR.GREEN, element.text, true)
end

function ColorConfiguratorDialog:onTextChangedBlue(element)
	self:onRGBElementChanged(ColorConfiguratorDialog.COLOR.BLUE, element.text, true)
end

function ColorConfiguratorDialog:onChangedSliderRed(value)
	self:onRGBElementChanged(ColorConfiguratorDialog.COLOR.RED, value, false)
end

function ColorConfiguratorDialog:onChangedSliderGreen(value)
	self:onRGBElementChanged(ColorConfiguratorDialog.COLOR.GREEN, value, false)
end

function ColorConfiguratorDialog:onChangedSliderBlue(value)
	self:onRGBElementChanged(ColorConfiguratorDialog.COLOR.BLUE, value, false)
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextRed(unicode)
	return self:onIsUnicodeAllowed(self.textInputRed, unicode)
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextGreen(unicode)
	return self:onIsUnicodeAllowed(self.textInputGreen, unicode)
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextBlue(unicode)
	return self:onIsUnicodeAllowed(self.textInputBlue, unicode)
end

function ColorConfiguratorDialog:onRGBElementChanged(id, value, textChanged, forceUpdate)
	local colorValue = tonumber(value) or 0
	local sliderElement = self.sliderElements[id]
	local textInputElement = self.textInputElements[id]

	if not forceUpdate then
		if textChanged then
			self:setSliderValueWithNoCallback(sliderElement, colorValue)
		else
			self:setTextWithNoCallback(textInputElement, tostring(colorValue))
			self:playSample(self.sliderSample)
		end
	else
		self:setSliderValueWithNoCallback(sliderElement, colorValue)
		self:setTextWithNoCallback(textInputElement, tostring(colorValue))
	end

	if self.isRGBMode then
		colorValue = ColorConfiguratorUtil.rgbToGiants(colorValue)
	end

	self.colors[id] = colorValue

	if not forceUpdate then
		self:updateHSVElements()
		self:updateSceneRender()
		self:updateTooltipLayout(false)
	end
end

function ColorConfiguratorDialog:getTypedText(element, unicode)
	local text = (element.cursorPosition > 1 and utf8Substr(element.text, 0, element.cursorPosition - 1) or "") .. unicodeToUtf8(unicode) .. (element.cursorPosition <= utf8Strlen(element.text) and utf8Substr(element.text, element.cursorPosition - 1) or "")
	local number = tonumber(text)

	return text, number
end

function ColorConfiguratorDialog:onIsUnicodeAllowed(element, unicode)
	local text, number = self:getTypedText(element, unicode)

	if number == nil or unicode == Input.KEY_space then
		return false
	end

	if self.isRGBMode then
		return not MathUtil.getIsOutOfBounds(number, 0, 255) and not string.startsWith(text, "02") and not string.startsWith(text, "01") and not string.startsWith(text, "00") and unicode ~= Input.KEY_period
	end

	return not MathUtil.getIsOutOfBounds(number, 0, 1) and not string.startsWith(text, "1.") and not string.startsWith(text, "01") and not string.startsWith(text, "00")
end

function ColorConfiguratorDialog:setValues(colors, hasMaterial, isModeChange)
	for id = 1, 3 do
		local color = colors[id]

		if self.isRGBMode then
			color = ColorConfiguratorUtil.giantsToRgb(color)
		end

		self:onRGBElementChanged(id, color, _, true)
	end

	local state = self:getMaterialState(colors[ColorConfiguratorDialog.COLOR.MATERIAL])

	if self.multiMaterialType.state ~= state then
		self.multiMaterialType:setState(state, true)
	end

	if not isModeChange then
		self:updateHSVElements()
		self:updateSceneRender()
		self:updateTooltipLayout(not hasMaterial, g_i18n:getText(ColorConfiguratorDialog.L10N_SYMBOL.NO_MATERIAL))
	end
end

function ColorConfiguratorDialog:verifyInputText(textInputElement, dismissal)
	local colorNumber = tonumber(textInputElement.text) or 0
	local maxValue = 1

	if self.isRGBMode then
		maxValue = 255
	end

	textInputElement:setText(tostring(MathUtil.clamp(colorNumber, 0, maxValue)))
end

function ColorConfiguratorDialog:updateSceneRender()
	local r, g, b, mat = unpack(self.colors)
	local isMetallic = ConfigurationUtil.isColorMetallic(mat)
	local element = self.pickedColor

	element:getDescendantByName("colorImage"):setImageColor(nil, r, g, b, 1)
	element:getDescendantByName("colorImage"):setVisible(not isMetallic)
	element:getDescendantByName("colorImageMetallic"):setImageColor(nil, r, g, b, 1)
	element:getDescendantByName("colorImageMetallic"):setVisible(isMetallic)

	self.sceneRender:updateShaderParameter(r, g, b, mat)
end

function ColorConfiguratorDialog:updateTooltipLayout(visible, text)
	if self.tooltipLayout:getIsVisible() ~= visible then
		self.tooltipLayout:setVisible(visible)
	end

	if text ~= nil then
		self.tooltipLayout.elements[1]:setText(text)
	end
end

function ColorConfiguratorDialog:getCanAddConfig()
	local r, g, b, mat = unpack(self.colors)
	local alreadyExists = false
	local storeItem = self.colorConfiguratorPickerDialog.target.storeItem
	local configName = self.colorConfiguratorPickerDialog.args.configName
	local baseConfigs = storeItem.configurations[configName]

	for _, config in pairs(baseConfigs) do
		if not config.isCustomConfig then
			local configMaterial = config.material
			local configColor = config.color

			if mat == configMaterial and ColorConfiguratorUtil.equals(configColor, {r, g, b}) then
				alreadyExists = true
				break
			end
		end
	end

	local addConfigsCount = 0

	if not alreadyExists then
		local filename = storeItem.xmlFilename
		local addConfigs = self.colorConfigurator:getAdditionalConfigs(filename, configName)

		if addConfigs ~= nil then
			addConfigsCount = #addConfigs

			for _, addColor in ipairs(addConfigs) do
				local configMaterial = addColor.material
				local configColor = addColor.color

				if mat == configMaterial and ColorConfiguratorUtil.equals(configColor, {r, g, b}) then
					alreadyExists = true
					break
				end
			end
		end
	end

	return alreadyExists, #baseConfigs + addConfigsCount >= ColorConfiguratorDialog.MAX_CONFIGS
end

function ColorConfiguratorDialog:getMaterialState(material)
	for i, mat in pairs(self.materials) do
		if mat.shaderId == material then
			return i
		end
	end

	return 1
end

function ColorConfiguratorDialog:onCopyColorValues()
	self.copiedColors = table.copy(self.colors)

	self:updateTooltipLayout(true, g_i18n:getText(ColorConfiguratorDialog.L10N_SYMBOL.COPY_COLOR))
	self:playSample(GuiSoundPlayer.SOUND_SAMPLES.CLICK)
end

function ColorConfiguratorDialog:onPasteColorValues()
	local colors = self.copiedColors

	if colors ~= nil then
		self:setValues(colors, true)
		self:updateTooltipLayout(true, g_i18n:getText(ColorConfiguratorDialog.L10N_SYMBOL.PASTE_COLOR))
		self:playSample(GuiSoundPlayer.SOUND_SAMPLES.CLICK)
	end
end

function ColorConfiguratorDialog:loadSettingsFromXMLFile(printToLog)
	local xmlFile = XMLFile.loadIfExists("settingsXML", self.settingsXMLFilename, "settings")

	if xmlFile ~= nil then
		local isRGBMode = xmlFile:getBool("settings.isRGBMode", self.isRGBMode)

		self.isRGBMode = isRGBMode
		self.savedMode = isRGBMode

		if printToLog then
			ColorConfiguratorUtil.printToLog(xmlFile)
		end

		xmlFile:delete()
	end
end

function ColorConfiguratorDialog:saveSettingsToXMLFile(printToLog)
	local xmlFile = XMLFile.create("settingsXML", self.settingsXMLFilename, "settings")

	if xmlFile ~= nil then
		xmlFile:setBool("settings.isRGBMode", self.isRGBMode)

		xmlFile:save(printToLog)
		xmlFile:delete()
	end
end

function ColorConfiguratorDialog:setRGBMode(isRGBMode)
	self.isRGBMode = isRGBMode

	local sliderMaxValue = 1
	local sliderStepSize = 0.003921
	local multiState = ColorConfiguratorDialog.MODE_GIANTS

	if isRGBMode then
		sliderMaxValue = 255
		sliderStepSize = 1
		multiState = ColorConfiguratorDialog.MODE_RGB
	end

	for _, elem in pairs(self.sliderElements) do
		elem.maxValue = sliderMaxValue
		elem.stepSize = sliderStepSize
	end

	if self.multiColorMode.state ~= multiState then
		self.multiColorMode:setState(multiState, true)
	end

	self:setValues(self.colors, true, true)
end

function ColorConfiguratorDialog:setTextWithNoCallback(textInputElement, text)
	local onTextChangedCallbackBackup = textInputElement.onTextChangedCallback

	textInputElement.onTextChangedCallback = NO_CALLBACK
	textInputElement:setText(text)
	textInputElement.onTextChangedCallback = onTextChangedCallbackBackup
end

function ColorConfiguratorDialog:setSliderValueWithNoCallback(sliderElement, value)
	local onChangedCallbackBackup = sliderElement.onChangedCallback

	sliderElement.onChangedCallback = NO_CALLBACK
	sliderElement:setValue(value)
	sliderElement.onChangedCallback = onChangedCallbackBackup
end

function ColorConfiguratorDialog:setPickerValuesWithNoCallback(pickerElement, value)
	local onChangedCallbackBackup = pickerElement.onChangedCallback

	pickerElement.onChangedCallback = NO_CALLBACK
	pickerElement:setValues(value)
	pickerElement.onChangedCallback = onChangedCallbackBackup
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextHue(unicode)
	local text, number = self:getTypedText(self.textInputHue, unicode)

	if number == nil or unicode == Input.KEY_space or unicode == Input.KEY_period or MathUtil.getIsOutOfBounds(number, 0, 360) or string.startsWith(text, "03") or string.startsWith(text, "02") or string.startsWith(text, "01") or string.startsWith(text, "00") then
		return false
	end

	return true
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextSatVal(element, unicode)
	local text, number = self:getTypedText(element, unicode)

	if number == nil or unicode == Input.KEY_space or unicode == Input.KEY_period or MathUtil.getIsOutOfBounds(number, 0, 100) or string.startsWith(text, "01") or string.startsWith(text, "00") then
		return false
	end

	return true
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextSaturation(unicode)
	return self:onIsUnicodeAllowedTextSatVal(self.textInputSaturation, unicode)
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextValue(unicode)
	return self:onIsUnicodeAllowedTextSatVal(self.textInputValue, unicode)
end

function ColorConfiguratorDialog:verifyInputTextHSV(textInputElement, maxValue, dismissal)
	local colorNumber = tonumber(textInputElement.text) or 0

	textInputElement:setText(tostring(MathUtil.clamp(colorNumber, 0, maxValue)))
end

function ColorConfiguratorDialog:onButtonPressedHue(element, dismissal)
	self:verifyInputTextHSV(element, 360, dismissal)
end

function ColorConfiguratorDialog:onButtonPressedSatVal(element, dismissal)
	self:verifyInputTextHSV(element, 100, dismissal)
end

function ColorConfiguratorDialog:updateHSVElements()
	local r, g, b = unpack(self.colors)
	local hsvColors = {ColorConfiguratorUtil.rgbToHsv(ColorConfiguratorUtil.giantsToRgb(r), ColorConfiguratorUtil.giantsToRgb(g), ColorConfiguratorUtil.giantsToRgb(b))}

	self:setPickerValuesWithNoCallback(self.colorPicker, hsvColors)

	for i, elem in pairs(self.textInputHSVElements) do
		self:setTextWithNoCallback(elem, tostring(hsvColors[i]))
	end
end

function ColorConfiguratorDialog:onChangedColorPicker(hsvColors, isSliderChange)
	self:onHSVElementChanged(hsvColors, false)
end

function ColorConfiguratorDialog:onTextChangedHue(element)
	local _, s, v = self.colorPicker:getValues()

	self:onHSVElementChanged({tonumber(element.text) or 0, s, v}, true)
end

function ColorConfiguratorDialog:onTextChangedSaturation(element)
	local h, _, v = self.colorPicker:getValues()

	self:onHSVElementChanged({h, tonumber(element.text) or 0, v}, true)
end

function ColorConfiguratorDialog:onTextChangedValue(element)
	local h, s, _ = self.colorPicker:getValues()

	self:onHSVElementChanged({h, s, tonumber(element.text) or 0}, true)
end

function ColorConfiguratorDialog:onHSVElementChanged(hsvColors, textChanged)
	if textChanged then
		self:setPickerValuesWithNoCallback(self.colorPicker, hsvColors)
	else
		self:playSample(self.sliderSample)

		for i, elem in pairs(self.textInputHSVElements) do
			self:setTextWithNoCallback(elem, tostring(hsvColors[i]))
		end
	end

	local r, g, b = ColorConfiguratorUtil.hsvToRgb(unpack(hsvColors))
	local giantsColors = {ColorConfiguratorUtil.rgbToGiants(r), ColorConfiguratorUtil.rgbToGiants(g), ColorConfiguratorUtil.rgbToGiants(b)}
	local elementsValues = {r, g, b}

	if not self.isRGBMode then
		elementsValues = giantsColors
	end

	for i = 1, 3 do
		self:setTextWithNoCallback(self.textInputElements[i], tostring(elementsValues[i]))
		self:setSliderValueWithNoCallback(self.sliderElements[i], elementsValues[i])

		self.colors[i] = giantsColors[i]
	end

	self:updateSceneRender()
	self:updateTooltipLayout(false)
end